@@ -332,7 +332,7 @@ class Vocabulary(object):
         :lang lang: Language
         """
 
-        missing = Counter(countries) if countries is not 'nogeo' else None
+        missing = Counter(countries) if countries != 'nogeo' else None
         rest = []
         dates = dates[::-1]
         while len(dates) and (len(rest) < n or n == -1):
@@ -470,7 +470,8 @@ class BagOfWords(SparseMatrix):
     :py:class:`text_models.vocabulary.Tokenize`
     
     :param tokens: Language (Ar|En|Es) or list of tokens
-    :type tokens: str|List
+    :type tokens: [str|List] 
+
     >>> from EvoMSA.tests.test_base import TWEETS
     >>> from microtc.utils import tweet_iterator
     >>> from text_models.vocabulary import BagOfWords
